package Site::SQLContainer;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#
use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use PObject;
@ISA = qw( PObject );

use Site::Page;
use Site::SQLField;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			table => undef,
			pkeys => undef,
			pkey => undef, # Just to catch the pkey method

			namedfields => undef,
			listfields => undef,
			boundfields => undef,

			opts => undef,
			dbh => undef,

			carry => undef,
			objectname => undef
	);

sub OBJ_NEW
{
	my $Self = shift;

	$Self -> { namedfields } = {};
	$Self -> { boundfields } = {};
	$Self -> { listfields } = [];

	$Self -> { carry } = {};
	$Self -> { pkeys } = [];

	if( ! defined( $Self -> { opts } ) )
	{
		# Kludge?
		$Self -> { opts } = $Site::Page::OPTS;
	}

	return( $Self );
}

sub addfield
{
	my $self = shift;
	my %Fields = @_;

	return( $self -> addfield_href( \%Fields ) );
}

sub addfield_href
{
	my $Self = shift;
	my $FixEm = shift;
	my $Field = Site::SQLField -> new( container => $Self,
						type => $FixEm -> { type } );

	for ( keys( %{ $FixEm } ) )
	{
		$Field -> $_( $FixEm -> { $_ } );
	}

	# Push the ref also into the listfields
	push( @{ $Self -> { listfields } }, $Field );
	return( $Field );
}

sub addfields
{
	my $Self = shift;

	# Just add the fields
	for ( @_ )
	{
		$Self -> addfield_href( $_ );
	}
}

sub _HTMLNAME
{
	my $Self = shift;
	return( join( ".", $Self -> table(), @{ $Self -> pkeys() } ) );
}

sub _ACTION
{
	# The name of my action
	my $Self = shift;
	return( $Self -> _HTMLNAME() . ".__.action" );
}

sub _TXPROTECT
{
	return( shift() -> _HTMLNAME() . ".__.txno" );
}

sub _SELECTFIELDS
{
	my $Self = shift;
	return( grep( /^.+$/, map( $_ -> _SQLSELECT(),
				@{ $Self -> listfields() } ) ) );

}

sub _MODIFYFIELDS
{
	my $Self = shift;
	# Return the field names of the fields to mod
	return( grep( /^.+$/, map( $_ -> _SQLUPDATE(),
				@{ $Self -> listfields() } ) ) );

}

sub addcarry
{
	# Add vars to the carryo
	my $Self = shift;
	my %Carry = @_;

	for( keys( %Carry ) )
	{
		$Self -> carry() -> { $_ } = $Carry{ $_ };
	}
}

# Quickie to set only 1 pkey
sub OBJ_HOOK_pkey
{
	my $Self = shift;
	$Self -> pkeys( [ shift() ] );
	return( $Self -> pkeys() -> [ -1 ] );
}

sub htmlname
{
	# LEGACY
	$_[0] -> _dmesg( "Deprecated function", -1 );
	return( shift() -> html_name( @_ ) );
}

sub html_name
{
	return( shift() -> named_field( @_ ) -> htmlname() );
}

sub getboundfield
{
	# LEGACY
	$_[0] -> _dmesg( "Deprecated function", -1 );
	return( shift() -> bound_field( @_ ) );
}

sub bound_field
{
	my $Self = shift;
	return( $Self -> boundfields() -> { shift() } );
}

sub getnamedfield
{
	# LEGACY
	$_[0] -> _dmesg( "Deprecated function", -1 );
	return( shift() -> named_field( @_ ) );
}

sub named_field
{
	my $Self = shift;
	return( $Self -> namedfields() -> { shift() } );
}

sub getfield
{
	return( shift() -> getnamedfield( shift() ) );
}

sub bind_sql_object
{
	# Takes an SQLObject subclass name and
	# returns a whatever based on it.  Ye!
	my $ClassType = shift;
	my $ObjType = shift;

	my( $Ret, %sp, %field, @PKeys, $Field, %FAT, $IsLocked );

	eval( "use $ObjType" );
	%sp = $ObjType -> _SQL_PROPERTIES();

	$Ret = $ClassType -> new();

	# Some copys
	$Ret -> table( $sp{ table } );
	$Ret -> dbh( $sp{ dbh } );

	# PKeys
	@PKeys = map( $_ -> { name }, @{ $sp{ pkeys } } );
	$Ret -> pkeys( \@PKeys );

	# Now the fields
	for( @{ $sp{ pkeys } }, @{ $sp{ fields } } )
	{
		if( ! $_ -> { insert }  )
		{
			$IsLocked = 1;
		}else
		{
			$IsLocked = 0;
		}

		if( $_ -> { fat } )
		{
			%FAT = %{ $_ -> { fat } };
		}else
		{
			%FAT = ();
		}

		$Field = $Ret -> addfield(
				name => $_ -> { name },
				boundto => $_ -> { name },
				locked => $IsLocked,
				enabled => ( ! $IsLocked ),
				%FAT );
	}

	$Ret -> objectname( $ObjType );
	return( $Ret );
}

sub defaultdbh
{
        shift();

        $AUTOFIELDS{ dbh } = shift();
}

sub defaultopts
{
        shift();

        $AUTOFIELDS{ opts } = shift();
}

sub _DELETEFIELD
{
	my $Self = shift;
	my $Field = shift;
	my( $Name, $Bound, @NewFields );

	# This is for fields to request to be removed from the
	# @FIELDS array
	$Bound = $Field -> boundto();
	$Name = $Field -> name();

	@NewFields = grep( $_ != $Field, @{ $Self -> listfields() } );
	$Self -> { listfields } = \@NewFields;

	if( $Bound )
	{
		delete( $Self -> boundfields() -> { $Bound } );
	}

	if( $Name )
	{
		delete( $Self -> namedfields() -> { $Name } );
	}
}

sub removeexceptnamed
{
	# LEGACY
	$_[0] -> _dmesg( "Deprecated function", -1 );
	return( shift() -> remove_except_named( @_ ) );
}

sub remove_except_named
{
	# Removes all but the listed fields
	my $Self = shift;
	my @ToKeep = @_;
	my( $name );

	for( @{ $Self -> listfields() } )
	{
		$name = $_ -> name();
		if( ! grep( $_ eq $name, @ToKeep ) )
		{
			$_ -> remove();
		}
	}
}

# Legacy support

1;

__END__

=head1 NAME

Site::SQLContainer - Base funcionality for forms/tables

=head1 SYNOPSIS

  use Site::SQLContainer;
  @ISA = qw( Site::SQLContainer );

=head1 DESCRIPTION

The SQLContainer is a base class for SQLForm and SQLTable, providing
general field and database handling code.

=head2 new

Creates a new object

=head2 dbh

This should be set with a valid DBI::DBD database handle, this
would even work:

  $f -> dbh( DBI -> connect( 'DBI:Pg:dbname=test' ) );

Though you might want to use a shared handle.

=head2 table

Sets the table name for this container. i.e. if you have a form
based on a table 'customers' you would set the table to 'customers'
and use addfield() or addfields() to place the fields in the form.

=head2 addfield

Use this to add a field to a container.  You need to specify the field
type here and if it's bound/named or anything like that.

  $f -> addfield(
			boundto => 'testid',
			type => 'text'
		);

The default type is text if omitted.  For more information about fields
and field types/options, see Site::SQLField.

=head2 addfields

Allows addition of multiple fields quickly.  Takes a bunch of hashrefs.

  $f -> addfields( {
			boundto => 'testid'
		}, {
			boundto => 'testvalue'
		}, {
			boundto => 'foreign_key',
			type => 'dropbox',
			rowsourcetype => 'query',
			rowsourcequery => 'select * from foreign_table'
		} );

=head2 addcarry

Lots of times you'll have a screen based on a particular ID or some
other kind of data and it needs to be present in all queries.  Use
the addcarry() method to have these passed without fully qualified
names within the container, since normally fields are passed around
by tablename.pkey.fieldname.  So if you need the customerid passed
back when the table re-sorts or the form submits or whatever, add
a carry like this:

  $f -> addcarry( customerid => $custid );

And it adds a HIDDEN field to whatever with NAME=customerid and VALUE=
$custid.

=head2 getnamedfield/getboundfield

These return the field named or bound to the argument.  If you need
to change a default or if the field is enabled, etc.  Most useful for
things created from a bind_sql_object() call where you might need to
set defaults different from those within the class.

=head2 bind_sql_object

Creates a container from an SQL object.  Sets the dbh(), adds the
fields along with their fat{} (field attributes) and sets the pkeys,
etc.

=head2 template

Assigns a template object to the container.  For more information
about templates in general, see Site::Template.  For more specific
information about templates for tables or forms, see SQLForm and
SQLTable.

=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).

=cut
